// Copyright (c) 2006, Rodrigo Braz Monteiro
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of the Aegisub Group nor the names of its contributors
//     may be used to endorse or promote products derived from this software
//     without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// -----------------------------------------------------------------------------
//
// AEGISUB
//
// Website: http://aegisub.cellosoft.com
// Contact: mailto:zeratul@cellosoft.com
//


///////////
// Headers
#include <vector>
#include <wx/wxprec.h>


///////////////////////////
// Hiragana/katakana entry
class KanaEntry {
public:
	wxString hiragana;
	wxString katakana;
	wxString hepburn;
	wxString group;
	int level;

	KanaEntry(wxString hira,wxString kata,wxString hep) {
		hiragana = hira;
		katakana = kata;
		hepburn = hep;
	}
};


///////////////////////////
// Hiragana/Katakana table
class KanaTable {
private:
	std::vector<KanaEntry> entries;
	void Insert(wchar_t *hira,wchar_t *kata,wchar_t *hep);
	void BeginGroup();
	wxString curGroup;
	int groups[2];
	int level;

public:
	KanaTable();
	~KanaTable();

	int GetNumberEntries(int level=-1) const;
	int GetLevels(int table) const;
	const KanaEntry &GetEntry(int i) const;
	const KanaEntry *FindByRomaji(wxString romaji) const;
	const KanaEntry *FindByKana(wxString kana) const;

	wxString KanaToRomaji(wxString kana,int type=1);
};
